pub struct RandomXVmCache {
pub seeds: VecDeque<(usize, [u8; 32])>,
pub vms: HashMap<usize, Arc<RandomXVm>>,
pub cached_vm: Option<([u8; 32], Arc<RandomXVm>)>,
}Expand description
The randomX VMs cache, keeps the VM needed to calculate the current block’s proof-of-work hash (if a VM is needed) and a couple more around this VM.
Fields
seeds: VecDeque<(usize, [u8; 32])>The top RX_SEEDS_CACHED RX seeds.
vms: HashMap<usize, Arc<RandomXVm>>The VMs for seeds (if after hf 12, otherwise this will be empty).
cached_vm: Option<([u8; 32], Arc<RandomXVm>)>A single cached VM that was given to us from a part of Cuprate.
Implementations
Sourceimpl RandomXVmCache
impl RandomXVmCache
pub async fn init_from_chain_height<D: Database + Clone>( chain_height: usize, hf: &HardFork, database: D, ) -> Result<Self, ContextCacheError>
Sourcepub fn add_vm(&mut self, vm: ([u8; 32], Arc<RandomXVm>))
pub fn add_vm(&mut self, vm: ([u8; 32], Arc<RandomXVm>))
Add a randomX VM to the cache, with the seed it was created with.
Sourcepub async fn get_alt_vm<D: Database>(
&self,
height: usize,
chain: Chain,
database: D,
) -> Result<Arc<RandomXVm>, ContextCacheError>
pub async fn get_alt_vm<D: Database>( &self, height: usize, chain: Chain, database: D, ) -> Result<Arc<RandomXVm>, ContextCacheError>
Creates a RX VM for an alt chain, looking at the main chain RX VMs to see if we can use one of them first.
Sourcepub async fn get_vms(&mut self) -> HashMap<usize, Arc<RandomXVm>>
pub async fn get_vms(&mut self) -> HashMap<usize, Arc<RandomXVm>>
Get the main-chain RandomX VMs.
Sourcepub async fn pop_blocks_main_chain<D: Database + Clone>(
&mut self,
new_height: usize,
database: D,
) -> Result<(), ContextCacheError>
pub async fn pop_blocks_main_chain<D: Database + Clone>( &mut self, new_height: usize, database: D, ) -> Result<(), ContextCacheError>
Removes all the RandomX VMs above the new_height.